/* SPDX-License-Identifier: LGPL-3.0-or-later */

#pragma once


#define VIRT_LINT_MAJOR 0
#define VIRT_LINT_MINOR 0
#define VIRT_LINT_PATCH 1


#include <stdarg.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>
#include <libvirt/libvirt.h>

typedef enum WarningDomain {
  /**
   * The problem lies inside of domain XML
   */
  Domain,
  /**
   * The problem lies on remote host
   */
  Node,
} WarningDomain;

typedef enum WarningLevel {
  /**
   * Critical error, domain won't start
   */
  Error,
  /**
   * Suboptimal domain configuration
   */
  Warning,
  /**
   * Domain configuration is okay, but can use tweaking
   */
  Notice,
} WarningLevel;

typedef struct VirtLint VirtLint;

typedef struct VirtLintError VirtLintError;

typedef struct CVirtLintWarning {
  char **tags;
  size_t ntags;
  enum WarningDomain domain;
  enum WarningLevel level;
  char *msg;
} CVirtLintWarning;

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

unsigned long virt_lint_version(void);

void virt_lint_string_free(char *string);

void virt_lint_error_free(struct VirtLintError **err);

char *virt_lint_error_get_message(const struct VirtLintError *err);

struct VirtLint *virt_lint_new(virConnectPtr conn);

void virt_lint_free(struct VirtLint *vl);

int virt_lint_capabilities_set(struct VirtLint *vl,
                               const char *capsxml,
                               struct VirtLintError **err);

int virt_lint_domain_capabilities_clear(struct VirtLint *vl, struct VirtLintError **err);

int virt_lint_domain_capabilities_add(struct VirtLint *vl,
                                      const char *domcapsxml,
                                      struct VirtLintError **err);

int virt_lint_validate(struct VirtLint *vl,
                       const char *domxml,
                       const char **tags,
                       size_t ntags,
                       bool error_on_no_connect,
                       struct VirtLintError **err);

ptrdiff_t virt_lint_list_tags(char ***tags, struct VirtLintError **err);

ptrdiff_t virt_lint_get_warnings(const struct VirtLint *vl,
                                 struct CVirtLintWarning **warnings,
                                 struct VirtLintError **err);

void virt_lint_warnings_free(struct CVirtLintWarning **warnings, ptrdiff_t *nwarnings);

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus
